[data-theme="light"] {
  --tcv-theme-blue: rgb(83, 160, 227);
  --tcv-theme-led: rgb(255, 150, 0);
  --tcv-font-color: #333;
  --tcv-bg-color: #fff;
  --tcv-bg-overlay-color: rgba(255, 255, 255, 0.7);
  --tcv-bg-highlight-color: #ddd;
  --tcv-bg-pressed-color: #eee;
  --tcv-bg-pressed-border-color: #bbb;
  --tcv-bg-tooltip-color: rgb(226, 238, 248);
  --tcv-x-color: red;
  --tcv-y-color: green;
  --tcv-z-color: blue;
  --tcv-shadow: rgba(0, 0, 0, 0.4);
  --tcv-menu-shadow: rgba(127, 127, 127, 0.4);
  --tcv-scrollbar-hover: rgba(0, 0, 0, 0.3);
  --tcv-scrollbar-active: rgba(0, 0, 0, 0.5);
  --tcv-scrollbar-thumb: rgba(0, 0, 0, 0.1);
  --tcv-dropdown-bg: #eee;
}

[data-theme="dark"] {
  --tcv-theme-blue: rgb(48, 142, 225);
  --tcv-theme-led: rgb(255, 230, 0);
  --tcv-font-color: #ddd;
  --tcv-bg-color: #444;
  --tcv-bg-overlay-color: rgba(68, 68, 68, 0.7);
  --tcv-bg-highlight-color: rgb(104, 104, 104);
  --tcv-bg-pressed-color: rgb(94, 94, 94);
  --tcv-bg-pressed-border-color: #999;
  --tcv-bg-tooltip-color: rgb(77, 109, 138);
  --tcv-x-color: rgb(255, 69, 0);
  --tcv-y-color: rgb(50, 205, 50);
  --tcv-z-color: rgb(59, 158, 255);
  --tcv-shadow: rgba(255, 255, 255, 0.6);
  --tcv-menu-shadow: rgba(127, 127, 127, 0.6);
  --tcv-scrollbar-hover: rgba(255, 255, 255, 0.4);
  --tcv-scrollbar-active: rgba(255, 255, 255, 0.6);
  --tcv-scrollbar-thumb: rgba(255, 255, 255, 0.2);
  --tcv-dropdown-bg: #333;
}

.tcv_none {
  display: none;
}
canvas {
  outline: none !important;
}

.tcv_cad_viewer {
  margin: 0;
  display: flex;
  flex-direction: column;
  font-family: sans-serif;
  -webkit-text-size-adjust: none;
  background-color: var(--tcv-bg-color);
  color: var(--tcv-font-color);
  margin: 4px;
  width: fit-content;
}

.tcv_cad_navigation {
  display: flex;
  flex-direction: column;
  z-index: 100;
}

.tcv_cad_body {
  display: flex;
  flex-direction: row;
}

.tcv_cad_view {
  position: relative;
  margin: 2px;
}

.tcv_cad_view_glass {
  position: absolute !important;
  left: 4px;
}

.tcv_cad_tree_toggles {
  text-align: right;
  margin-right: 0px;
  margin-bottom: 3px;
}

.tcv_cad_tree {
  display: flex;
  flex-direction: column;
  width: 240px;
  height: 250px;
  border-style: solid;
  border-width: 1px;
  overflow: hidden;
  border-top: 1px solid lightgray;
}

.tcv_cad_tree_glass {
  border: none !important;
  height: fit-content;
}

.tcv_cad_tree_glass:hover {
  background-color: var(--tcv-bg-overlay-color);
}

.tcv_cad_info {
  width: 240px;
  height: 146px;
  font-family: sans-serif;
  font-size: 12px;
  border-style: solid;
  border-width: 1px;
  overflow: hidden;
}

.tcv_cad_info_wrapper {
  display: flex;
  flex-direction: column;
}

.tcv_cad_info_toggler {
  vertical-align: middle;
}

.tcv_toggle_info_wrapper {
  display: none;
  margin-left: 4px;
}

.tcv_info_label {
  font-size: 12px;
  font-family: sans-serif;
  font-weight: bold;
  margin-left: 4px;
  padding-bottom: 4px;
  vertical-align: top;
  user-select: none;
}

.tcv_cad_info_glass {
  display: none;
  background: var(--tcv-bg-overlay-color);
}

.tcv_small_info_btn {
  width: 24px !important;
  height: 14px !important;
  font-size: 12px !important;
  color: var(--tcv-font-color);
  border: 1px solid var(--tcv-bg-overlay-color);
  font-family: "Courier New", Courier, monospace;
  font-weight: bold;
  margin-left: 2px;
}

.tcv_tcv_info_table {
  display: block;
  overflow-x: visible;
  white-space: nowrap;
  font-size: 14px;
  border-spacing: 0px;
}

.tcv_info_header {
  margin: 1px 0px 6px 8px;
  font-weight: bold;
}

.tcv_small_table {
  line-height: 14px;
}

.tcv_small_table_row>td,
th {
  padding-left: 6px;
}

.tcv_info_line {
  margin: 4px;
}

.tcv_info_row {
  vertical-align: top !important;
}

.tcv_info_num {
  color: #aaa;
  white-space: nowrap;
  margin: 0px;
}

.tcv_info_red {
  color: var(--tcv-x-color);
}

.tcv_info_green {
  color: var(--tcv-y-color);
}

.tcv_info_blue {
  color: var(--tcv-z-color);
}

.tcv_visible {
  display: block;
}

.tcv_unvisible {
  display: none;
}

.tcv_cad_toolbar {
  background: var(--tcv-bg-overlay-color);
}

.tcv_cad_inset {
  width: 80px;
  height: 80px;
  background-color: transparent;
  border: none;
  margin: 0;
  padding: 0px;
  position: absolute;
  left: 0px;
  bottom: 0px;
  z-index: 100;
}

.tcv_animation_label {
  vertical-align: middle;
  margin-left: 5px;
}

.tcv_tools_label {
  vertical-align: middle;
  margin-left: 5px;
}

.tcv_animation_slider {
  width: 120px !important;
}

.tcv_cad_animation {
  height: 36px;
  background-color: var(--tcv-bg-overlay-color);
  border: none;
  margin: 0;
  padding: 0px;
  position: absolute;
  right: 0px;
  bottom: 0px;
  z-index: 100;
}

.tcv_cad_tools {
  height: 38px;
  background-color: var(--tcv-bg-overlay-color);
  border: none;
  margin: 0;
  padding: 0px;
  position: absolute;
  right: 0px;
  bottom: 0px;
  z-index: 100;
}

.tcv_help {
  outline: none;
}

.tcv_cad_help {
  /* height: 36px; */
  background-color: var(--tcv-bg-color);
  border: none;
  margin: 0;
  padding: 0px;
  position: absolute;
  right: 0px;
  bottom: 0px;
  z-index: 100;
  display: none;
  font-size: 12px;
}

.tcv_cad_help_layout {
  border-collapse: separate;
  border-spacing: 4px;
}

.tcv_round {
  box-sizing: border-box;
  /* -moz-box-sizing: border-box;
    -webkit-box-sizing: border-box; */
  border-style: solid;
  border-width: 1px;
  border-radius: 6px;
  border-color: lightgray;
  margin: 2px;
  padding: 2px;
}

.tcv_align_right {
  float: right;
  margin-right: 4px;
}

.tcv_box_content {
  overflow: scroll;
  height: 100%;
  margin: 0px 0px 0px 4px;
  padding-top: 2px;
}

.tcv_btn {
  width: 30px;
  height: 30px;
  border: 0px;
  display: inline-block;
  vertical-align: middle;
  background-color: transparent;
  background-repeat: no-repeat;
  background-position: center;
  padding: 0px;
}

.tcv_button_frame {
  width: fit-content;
  height: fit-content;
  display: inline-block;
  border: 1px solid transparent;
  margin-left: 1px;
  margin-right: 1px;
}

.tcv_btn_highlight {
  border-bottom: 2px solid var(--tcv-theme-blue);
  border-radius: 6px;
}

.tcv_btn_click {
  width: 8px;
  height: 29px;
  margin-left: 12px;
  position: absolute;
  border-bottom: 3px inset var(--tcv-theme-led);
}

.tcv_btn_click2 {
  border: 1px solid var(--tcv-theme-blue);
  border-radius: 6px;
  background-color: var(--tcv-bg-pressed-color);
}

.tcv_separator {
  width: 14px;
  border-left: 1px solid lightgray;
  margin-left: 5px;
  padding-right: 5px;
  padding-bottom: 6px;
  padding-top: 3px;
}

.tcv_small_btn {
  width: 14px !important;
  height: 14px !important;
  font-size: 11px !important;
  color: var(--tcv-font-color);
  /* border: 1px solid var(--tcv-bg-overlay-color); */
}

.tcv_small_btn:hover {
  background-color: var(--tcv-bg-overlay-color);
}

.tcv_btn:hover {
  box-shadow: 1px px 1px 0 var(--tcv-shadow);
  background-color: var(--tcv-bg-highlight-color);
  border-radius: 6px;
}

.tcv_btn:active {
  box-shadow: none;
}

.tcv_tabnav {
  width: 100%;
  padding: 0px;
  display: flex;
  flex-wrap: nowrap;
  align-items: stretch;
}

input[type="button"] {
  -webkit-appearance: none;
  -moz-appearance: none;
  appearance: none;
  border-radius: 0;
  outline: none;
}

.tcv_tab {
  width: 50%;
  height: 24px;
  display: inline-block;
  padding: 0px;
  border: none;
  /* margin-bottom: 6px; */
  background-color: transparent;
  color: var(--tcv-font-color);
  font-size: 14px;
}

.tcv_tab:hover {
  background-color: var(--tcv-bg-highlight-color);
  font-weight: bold;
}

.tcv_tab-left {
  border-right: 1px solid lightgray;
}

.tcv_tab-right {
  border-left: 1px solid lightgray;
}

.tcv_tab-selected {
  border-bottom: none;
  font-weight: bold;
}

.tcv_tab-unselected {
  border-bottom: 1px solid lightgray;
  font-weight: normal;
}

.tcv_tab-disabled {
  color: var(--tcv-bg-pressed-color);
}

.tcv_vertical-center {
  margin: 0;
  position: absolute;
  top: 50%;
  -ms-transform: translateY(-50%);
  transform: translateY(-50%);
}

.tcv_label {
  font-family: sans-serif;
  font-size: 14px;
  margin-left: 4px;
  margin-right: 4px;
  padding-bottom: 4px;
  vertical-align: middle;
  user-select: none;
}

.tcv_check {
  vertical-align: middle;
  margin-right: 2px;
  margin-left: 6px;
}

.tcv_more-dropdown {
  position: relative;
  display: inline-block;
}

.tcv_more-btn {
  border: 0px;
  background-color: transparent;
  font-size: 14px;
  color: var(--tcv-font-color);
}

.tcv_dropdown-content {
  display: none;
  position: absolute;
  box-shadow: 0px 4px 8px 0px var(--tcv-menu-shadow);
  z-index: 200;
  background-color: var(--tcv-bg-color);
  padding-top: 2px;
  padding-bottom: 2px;
  margin-top: 0px;
}

.tcv_dropdown-entry {
  margin-top: 6px;
  margin-bottom: 6px;
}

.tcv_more_icon {
  font-size: 10px;
  margin-left: 4px;
}

.tcv_more-content {
  width: 115px;
}

.tcv_more-dropdown:hover .tcv_more-content {
  display: block;
}

.tcv_grid-dropdown {
  position: relative;
  display: inline-block;
}

.tcv_grid-content {
  width: 60px;
}

.tcv_grid-dropdown:hover .tcv_grid-content {
  display: block;
}

.tcv_slider_group {
  margin-bottom: 10px;
}

.tcv_material_ambientlight {
  margin-top: 4px;
}

.tcv_cad_clip_container {
  margin-top: 0px;
  overflow: hidden;
}

.tcv_cad_material_container {
  overflow: hidden;
}

.tcv_clip_checks {
  margin-top: 20px;
}

.tcv_clip_input {
  width: 25%;
  border: 1px solid #d3d3d3;
  background-color: var(--tcv-bg-overlay-color);
  color: var(--tcv-font-color);
}

.tcv_clip_slider {
  -webkit-appearance: none;
  appearance: none;
  width: 60%;
  height: 5px;
  border-radius: 2px;
  background: #d3d3d3;
  outline: none;
  opacity: 0.7;
  -webkit-transition: 0.2s;
  transition: opacity 0.2s;
  vertical-align: middle;
}

.tcv_clip_slider::-webkit-slider-thumb {
  -webkit-appearance: none;
  width: 12px;
  height: 12px;
  border-radius: 50%;
  background: #53a0e3;
  cursor: pointer;
  border: 1px solid #48627a;
}

.tcv_clip_slider::-moz-range-thumb {
  width: 12px;
  height: 12px;
  border-radius: 50%;
  background: #53a0e3;
  cursor: pointer;
  border: 1px solid #48627a;
}

.tcv_clip_slider:hover {
  opacity: 1;
}

.tcv_lbl_norm_plane1 {
  color: var(--tcv-x-color);
}

.tcv_lbl_norm_plane2 {
  color: var(--tcv-y-color);
}

.tcv_lbl_norm_plane3 {
  color: var(--tcv-z-color);
}

/*  Scrollbar styling for Chrome like browsers */

.tcv_mac-scrollbar::-webkit-scrollbar {
  width: 5px !important;
  height: 5px !important;
}

.tcv_mac-scrollbar::-webkit-scrollbar-thumb {
  background-color: var(--tcv-scrollbar-thumb) !important;
  border-radius: 100px !important;
}

.tcv_mac-scrollbar::-webkit-scrollbar-track {
  background-color: transparent !important;
}

.tcv_mac-scrollbar::-webkit-scrollbar-piece {
  background-color: transparent !important;
}

.tcv_mac-scrollbar::-webkit-scrollbar-corner {
  background-color: transparent !important;
}

.tcv_mac-scrollbar::-webkit-scrollbar-thumb:hover {
  background-color: var(--tcv-scrollbar-hover) !important;
}

.tcv_mac-scrollbar::-webkit-scrollbar-thumb:active {
  background-color: var(--tcv-scrollbar-hover) !important;
}

.tcv_mac-scrollbar .widget-html-content {
  overflow-x: visible;
  overflow-y: visible;
}

/* Scrollbar styling for Firefox Browser */

.tcv_scroller {
  scrollbar-width: thin;
  scrollbar-color: var(--tcv-scrollbar-thumb) transparent;
  background-color: transparent !important;
}

.tcv_scroller:hover {
  scrollbar-width: thin;
  scrollbar-color: var(--tcv-scrollbar-hover) transparent;
}

/* tooltips */
[data-tooltip]:before {
  position: absolute;
  pointer-events: none;
  content: attr(data-tooltip);
  white-space: nowrap;
  opacity: 0;
  font-size: 11px;
  font-family: sans-serif;
  transition: opacity 0.5s;
  z-index: 100;
  color: var(--tcv-font-color);
  background-color: var(--tcv-bg-tooltip-color);
  border: solid 1px var(--tcv-bg-tooltip-color);
  border-radius: 2px;
  margin-top: 2.5em;
  padding: 2px;
  margin-left: 24px;
  z-index: 200;
}

[data-tooltip]:hover:before {
  opacity: 1;
}

.tcv_material_info {
  font-size: 12px;
}

.tcv_filter_menu {
  position: relative;
}

.tcv_filter_dropdown {
  display: none;
}

.tcv_filter_icon {
  font-size: 12px;
  margin-left: 4px;
}

.tcv_shape_filter {
  color: var(--tcv-font-color);
  font-size: 13px;
  border: none;
  float: right;
  position: absolute;
  right: 0px;
  width: auto;
}

.tcv_shape_filter:hover {
  cursor: pointer;
}

.tcv_filter_dropdown_value {
  margin-left: 2px;
  margin-right: 2px;
  margin-top: 1px;
}

.tcv_filter_value {
  width: 3em;
  padding-left: 3px;
  display: inline-block;
}

.tcv_filter_dropdown_active {
  display: block;
}

.tcv_filter_dropdown_value:hover {
  cursor: pointer;
  background-color: var(--tcv-bg-pressed-color);
}

.tcv_filter_dropdown_inactive {
  display: none;
}

.tcv_filter_content {
  background-color: var(--tcv-bg-overlay-color);
}


.tv-tree-node {
  display: flex;
  flex-direction: column;
}

.tcv_toplevel {
  list-style-type: none;
  padding-inline-start: 0px;
  margin-block-start: 0px !important;
  margin-block-end: 0px !important;
}

.tv-node-content {
  display: flex;
  align-items: center;
  padding: 2px 0px 2px 0px;
  height: 16px;
  min-width: max-content;
}

.tv-nav-marker {
  display: inline-block;
  width: 16px;
  height: 16px;
  margin-right: 1px;
  cursor: pointer;
  vertical-align: middle;
  flex-shrink: 0;
}

.tv-nav-marker svg {
  width: 100%;
  height: 100%;
}

.tv-icon {
  display: inline-block;
  width: 24px;
  height: 24px;
  margin-right: 2px;
  vertical-align: middle;
  flex-shrink: 0;
}

.tv-pointer {
  cursor: pointer;
}

.tv-icon svg {
  width: 100%;
  height: 100%;
}

.tv-node-label {
  cursor: pointer;
  margin-left: 4px;
  font-size: 14px;
  font-family: sans-serif;
}

.tv-node-label-highlight {
  color: rgb(112, 160, 220);
}

.tv-children {
  margin-left: 20px;
}

/* MEASURE PANEL */


.tcv_panel {
    display: inline-block;
    text-align: center;
    position: absolute;
    background-color: var(--tcv-bg-overlay-color);
    user-select: none;
    font-family: sans-serif;
    color: var(--tcv-font-color);
    padding-left: 3px;
    padding-right: 3px;
}

.tcv_properties_table {
    border-collapse: collapse;
}

.tcv_measure_header {
    text-align: center;
    font-weight: bold;
    font-size: 16px;
    border-bottom: 1px solid lightgray;
    padding-top: 3px;
    padding-bottom: 3px;
}

.tcv_measure_subheader {
    text-align: center;
    font-style: italic;
    font-size: 14px;
    border-bottom: 1px solid lightgray;
    padding-top: 2px;
    padding-bottom: 2px;
}

.tcv_measure_cell_header {
    border-right: 1px solid lightgray;
}

.tcv_measure_row_line {
    border-bottom: 1px solid lightgray;
}

.tcv_measure_cell {
    padding: 3px 5px 31px 5px;
}

.tcv_vertex_cell {
    text-align: center;
    border-right: 1px solid lightgray;
    padding: 3px 5px 31px 5px;
}

.tcv_vertex_cell:last-child {
    border-right: none;
}

.tcv_props_cell {
    vertical-align: middle;
    font-size: 14px;
    font-family: sans-serif;
    border-bottom: 1px solid lightgray;
}

.tcv_props_header {
    text-align: left;
    padding: 3px 5px 3px 5px;
    border-right: 1px solid lightgray;
}

.tcv_measure_cell {
    text-align: left;
    padding: 3px 5px 3px 5px;
    font-size: 14px;
}

.tcv_measure_val {
    text-align: right;
    padding: 3px 5px 3px 5px;
    font-size: 14px;
}

.tcv_vertex_cell {
    text-align: center;
    border-right: 1px solid lightgray;
    padding: 3px 5px 3px 5px;
    font-size: 14px;
}

.tcv_xtitle {
    color: var(--tcv-x-color);
}

.tcv_ytitle {
    color: var(--tcv-y-color);
}

.tcv_ztitle {
    color: var(--tcv-z-color);
}

.tcv_angle {
    font-size: 14px;
    font-family: sans-serif;
}

.tcv_angle_val {
    text-align: right;
}